import arcpy, math, os
import matplotlib.pyplot as plt
import numpy as np
import encodings

class Toolbox(object):
    def __init__(self):
        self.label = "Spatial Measures"
        self.alias = "Spatial Measures"
        # List of tool classes associated with this toolbox
        self.tools = [Exposure, Concentration, Evenness, LocationQuotient, GiniAndLorenz, Centralization, Clustering, Neighborhood,Segregation]

# ---------==========EXPOSURE category (isolation, interaction)==========---------
class Exposure(object):
    def __init__(self):
        self.label = "Exposure"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_indice = arcpy.Parameter(
			displayName="Select which index you want to calculate",
			name="in_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        in_indice.filter.type = "ValueList"
        in_indice.filter.list=["Isolation", "Interaction"]

        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="First minority population field",
			name="st_minority_group",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Second minority population field",
			name="nd_minority_group",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        var3 = arcpy.Parameter(
			displayName="Population field",
			name="population",
			datatype="String",
			parameterType="Required",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Exposure"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_indice, in_features, var1, var2, var3, field_out, out_features]
        return params

    def isLicensed(self):
        return True

    def updateParameters(self, parameters):
        in_indice = parameters[0].valueAsText
        if in_indice == "Isolation":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = False
            parameters[3].value = ""
            parameters[4].enabled = True
        elif in_indice == "Interaction":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = True
            parameters[4].enabled = True
        else:
            parameters[1].enabled = False
            parameters[2].enabled = False
            parameters[3].enabled = False
            parameters[4].enabled = False
        if parameters[1].value is None:
            parameters[2].filter.list = []
            parameters[2].value = ""
            parameters[3].filter.list = []
            parameters[3].value = ""
            parameters[4].filter.list = []
            parameters[4].value = ""
        #code for sorting same values of input features
        if parameters[1].value:
            in_features = parameters[1].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
            parameters[3].filter.type = "ValueList"
            parameters[3].filter.list = sorted(existing)
            parameters[4].filter.type = "ValueList"
            parameters[4].filter.list = sorted(existing)
        return

    def updateMessages(self, parameters):
        if len(parameters[5].valueAsText) > 10:
            parameters[5].setErrorMessage("The length of Field Name must not be larger than 10")
        if parameters[0].valueAsText == "Interaction":
            if parameters[3].value is None:
                parameters[3].setErrorMessage("'Second minority population field' must be filled!")
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_indice = parameters[0].valueAsText
        in_features = parameters[1].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[2].valueAsText # first minority
        var2 = parameters[3].valueAsText # second minority
        var3 = parameters[4].valueAsText # whole population

        field_out = parameters[5].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            xi = 0 # first minority in region
            yi = 0 # second minority in region
            ti = 0 # population in region
            X = 0 # total minority population
            pjm = 0 # index
            # ------========Isolation========------
            if in_indice == "Isolation":
                arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "IsolationIndex", "NULLABLE")
                # calculation of total minority population
                try:
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        valueOfField = row.getValue(var1)
                        X = X + valueOfField
                # calculation of index
                    rows2 = arcpy.SearchCursor(in_features[p])
                    for row2 in rows2:
                        xi = row2.getValue(var1)
                        ti = row2.getValue(var3)
                        if ti == 0 or X == 0: continue
                        else:
                            pjm = pjm + (xi/X)*(xi/ti)
                    arcpy.AddMessage("Isolation index is: "+  str(pjm) + " ("+str(in_features[p])+")")
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(pjm), "PYTHON_9.3")
                    del rows, row, rows2, row2
            # ------========Interaction========------
            elif in_indice == "Interaction":
                arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "InteractionIndex", "NULLABLE")
                # calculation of total minority population
                try:
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        valueOfField = row.getValue(var1)
                        X = X + valueOfField
                # calculation of index
                    rows2 = arcpy.SearchCursor(in_features[p])
                    for row2 in rows2:
                        xi = row2.getValue(var1)
                        yi = row2.getValue(var2)
                        ti = row2.getValue(var3)
                        if ti == 0 or X == 0: continue
                        else:
                            pjm = pjm + (xi/X)*(yi/ti)
                    arcpy.AddMessage("Interaction index is: "+ str(pjm) + " ("+str(in_features[p])+")")
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(pjm), "PYTHON_9.3")
                    del rows, row, rows2, row2
        return

# --------============CONCENTRATION category (concentration)============--------
class Concentration(object):
    def __init__(self):
        self.label = "Concentration"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_pop_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Concetr"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_features, var1, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        #code for sorting same values of input features
        if parameters[0].value:
            in_features = parameters[0].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[1].filter.type = "ValueList"
            parameters[1].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[2].valueAsText) > 10:
            parameters[2].setErrorMessage("The length of Field Name must not be larger than 10")
        return
    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_features = parameters[0].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[1].valueAsText
        field_out = parameters[2].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            arcpy.DeleteField_management(in_features[p], field_name)
            arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "ConcentrationIndex", "NULLABLE")

            xi = 0 # size of minority in regionu
            x = 0 # total minority population in area
            ai = 0 # land area of region
            a = 0 # total land area
            cjm = 0 # Concentration Index

            area_r = "qwertyasd"
            arcpy.DeleteField_management(in_features[p], area_r)
            arcpy.AddField_management(in_features[p], area_r, "DOUBLE", "","", "", "area", "NULLABLE")
            arcpy.CalculateField_management(in_features[p], area_r, "!SHAPE.AREA@SQUAREMETERS!", "PYTHON_9.3")
            # calculation of total population and area
            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    valueOfField = row.getValue(var1)
                    valueOfField2 = row.getValue(area_r)
                    x = x + valueOfField
                    a = a + valueOfField2
            # calculation of index
                rows2 = arcpy.SearchCursor(in_features[p])
                for row2 in rows2:
                    xi = row2.getValue(var1)
                    ai = row2.getValue(area_r)
                    cjm = cjm + abs((xi/x)-(ai/a))
                cjm = cjm/2
                arcpy.AddMessage("Concentration index is: "+  str(cjm) + " ("+str(in_features[p])+")")
            except: raise
            finally:
                arcpy.CalculateField_management(in_features[p], field_name, str(cjm), "PYTHON_9.3")
                del row, rows, row2, rows2
                arcpy.DeleteField_management(in_features[p], area_r)

        return
# --------===========EVENNESS category (Dissimilarity, Entropy)===========--------
class Evenness(object):
    def __init__(self):
        self.label = "Evenness"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_indice = arcpy.Parameter(
			displayName="Select which index you want to calculate",
			name="in_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        in_indice.filter.type = "ValueList"
        in_indice.filter.list=["Dissimilarity", "Entropy"]

        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="First population field",
			name="first_population_field",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Second population field",
			name="second_population_field",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        var3 = arcpy.Parameter(
			displayName="Input Fields",
			name="input_field",
			datatype="String",
			parameterType="Optional",
			direction="Input",
            multiValue=True)

        tot = arcpy.Parameter(
			displayName="Total population field",
			name="population",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Evenness"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_indice ,in_features, var1, var2, var3, tot, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        in_indice = parameters[0].valueAsText
        if in_indice == "Dissimilarity":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = True
            parameters[4].enabled = False
            parameters[4].value = ""
            parameters[5].enabled = False
            parameters[5].value = ""
        elif in_indice == "Entropy":
            parameters[1].enabled = True
            parameters[2].enabled = False
            parameters[2].value = ""
            parameters[3].enabled = False
            parameters[3].value = ""
            parameters[4].enabled = True
            parameters[5].enabled = True
        else:
            parameters[1].enabled = False
            parameters[2].enabled = False
            parameters[3].enabled = False
            parameters[4].enabled = False
            parameters[5].enabled = False
        if parameters[1].value:
            in_features = parameters[1].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
            parameters[3].filter.type = "ValueList"
            parameters[3].filter.list = sorted(existing)
            parameters[4].filter.type = "ValueList"
            parameters[4].filter.list = sorted(existing)
            parameters[5].filter.type = "ValueList"
            parameters[5].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[6].valueAsText) > 10:
            parameters[6].setErrorMessage("The length of Field Name must not be larger than 10")
        if parameters[0].valueAsText == "Dissimilarity":
            if parameters[2].value is None:
                parameters[2].setErrorMessage("'First population field' must be filled!")
            if parameters[3].value is None:
                parameters[3].setErrorMessage("'Second population field' must be filled!")
        var3 = parameters[4].valueAsText
        if parameters[4].value:
            if len(var3.split(";")) < 2:
                parameters[4].setErrorMessage("'Input Fields' requires two or more fields!")
        if parameters[0].valueAsText == "Entropy":
            if parameters[5].value is None:
                parameters[5].setErrorMessage("'Total population field' must be filled!")
        return
    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_indice = parameters[0].valueAsText
        in_features = parameters[1].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[2].valueAsText # first population
        var2 = parameters[3].valueAsText # second population
        var3 = parameters[4].valueAsText # multi field
        tot = parameters[5].valueAsText # total pop
        field_out = parameters[6].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            # ------========Dissimilarity========------
            if in_indice == "Dissimilarity":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "DissimilarityIndex", "NULLABLE")
                pig = 0 # population group g in i
                pih = 0 # population group h in i
                pg = 0 # total population g
                ph = 0 # total population h
                delta = 0 # Dissimilarity index
                # calculation of total populations
                try:
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        valueOfField = row.getValue(var1)
                        valueOfField2 = row.getValue(var2)
                        pg = pg + valueOfField
                        ph = ph + valueOfField2
                # calculation of index
                    rows2 = arcpy.SearchCursor(in_features[p])
                    for row2 in rows2:
                        pig = row2.getValue(var1)
                        pih = row2.getValue(var2)
                        delta = delta + abs((pig/pg)-(pih/ph))
                    delta = delta/2
                    arcpy.AddMessage("Dissimilarity index is: "+  str(delta) + " ("+str(in_features[p])+")")
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(delta), "PYTHON_9.3")
                    del rows, row, rows2, row2
            # ------========Entropy========------
            elif in_indice == "Entropy":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "EntropyIndex", "NULLABLE")

                Sumaai = [] # sum in every input's field
                Xa = 0 # share for the population
                Xj = 0 # share for the population of entire area in each category of the variable studied
                E = 0 # total area's entropy score
                Ei = 0 # subunits area's entropy scores
                T = 0 # total area population
                ti = 0 # total population in subunit
                H = 0 # calculated index
                fields = var3.split(";")
                try:
                    # 1. and 3. step
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        valueof = row.getValue(tot)
                        T = T + valueof
                    del row, rows
                    for i in range(len(fields)):
                        rows2 = arcpy.SearchCursor(in_features[p])
                        j = 0
                        for row2 in rows2:
                            valueOfField = row2.getValue(fields[i])
                            j = j + valueOfField
                        Sumaai.append(j)
                        del rows2, row2
                    for i in range(len(fields)):
                        if T == 0: continue
                        else:
                            Xa = (Sumaai[i])/T
                            if Xa == 0: continue
                            else:
                                E = E + (Xa*math.log(1/Xa))
                    # 2. and 4. step
                    rows3 = arcpy.SearchCursor(in_features[p])
                    for row3 in rows3:
                        Ei = 0
                        for i in range(len(fields)):
                            ti = row3.getValue(tot)
                            xi = row3.getValue(fields[i])
                            if ti == 0: continue
                            else:
                                Xj = xi/ti
                                if Xj == 0: continue
                                else:
                                    Ei = Ei + (Xj*math.log(1/Xj))
                        ti = row3.getValue(tot)
                        if (E*T) == 0: continue
                        else:
                            H = H + ((ti*(E-Ei))/(E*T))
                    del row3, rows3
                except: raise
                finally:
                    arcpy.AddMessage("Entropy index is: "+  str(H) + " ("+str(in_features[p])+")")
                    arcpy.CalculateField_management(in_features[p], field_name, str(H), "PYTHON_9.3")
        return

# -------------==================Location Quotient==================-------------
class LocationQuotient(object):
    def __init__(self):
        self.label = "Location Quotient"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_pop_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Population field",
			name="population_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "LocQ"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_features, var1, var2, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        if parameters[0].value:
            in_features = parameters[0].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[1].filter.type = "ValueList"
            parameters[1].filter.list = sorted(existing)
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[3].valueAsText) > 10:
            parameters[3].setErrorMessage("The length of Field Name must not be larger than 10")
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_features = parameters[0].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[1].valueAsText # minority
        var2 = parameters[2].valueAsText # population
        field_out = parameters[3].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            arcpy.DeleteField_management(in_features[p], field_name)
            arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "LocationQuotient", "NULLABLE")

            xi = 0 # total minority population in region
            X = 0 # total population in region
            yi = 0 # total minority population in area
            Y = 0 # total population in area
            LQ = 0 # localization quotient

            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    valueOfField = row.getValue(var1)
                    valueOfField2 = row.getValue(var2)
                    yi = yi + valueOfField
                    Y = Y + valueOfField2
                rows2 = arcpy.UpdateCursor(in_features[p])
                for row2 in rows2:
                    LQ = 0
                    xi = row2.getValue(var1)
                    X = row2.getValue(var2)
                    if X == 0 or (yi/Y) == 0: continue
                    else:
                        LQ = (xi/X)/(yi/Y)
                    row2.setValue(field_name, LQ)
                    rows2.updateRow(row2)
                arcpy.AddMessage(str(in_features[p]) + " - Completed")
            except: raise
            finally:
                    del row,rows,row2, rows2
        return

# -----------================Gini Ratio & Lorenz curve================-----------
class GiniAndLorenz(object):
    def __init__(self):
        self.label = "Gini Ratio & Lorenz curve"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="Static field",
			name="static_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Second variables",
			name="second_variables",
			datatype="String",
			parameterType="Required",
			direction="Input",
            multiValue=True)

        out_graph = arcpy.Parameter(
			displayName="Output graph (Lorenz curve) into .png",
			name="out_graph",
			datatype="File",
			parameterType="Optional",
			direction="Output")
        out_graph.parameterDependencies=[in_features.name]
        out_graph.value = ""

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "GiniInd"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_features, var1, var2, out_graph, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        if parameters[0].value:
            in_features = parameters[0].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[1].filter.type = "ValueList"
            parameters[1].filter.list = sorted(existing)
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
        if parameters[3].value is None:
            return
        else:
            out_graph = str(parameters[3].valueAsText)
            out_graph = out_graph.replace(".","")
            parameters[3].value = out_graph
        return

    def updateMessages(self, parameters):
        if len(parameters[4].valueAsText) > 10:
            parameters[4].setErrorMessage("The length of Field Name must not be larger than 10")
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_features = parameters[0].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[1].valueAsText # Static variable
        var2 = str(parameters[2].valueAsText) # Second variables
        graph_de = parameters[3].valueAsText # out graph
        field_out = parameters[4].valueAsText # name of output field
        dens_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            out_graph = graph_de+str(p)
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            arcpy.DeleteField_management(in_features[p], dens_name)
            arcpy.AddField_management(in_features[p], dens_name, "DOUBLE", "","", "", "Density", "NULLABLE")
            stac_all, sec_all = 0, 0
            GR = 0 # gini ratio
            if parameters[3].value:
                plt.axis([-0.05,1.05,-0.05,1.05])
                plt.plot([0,1], [0,1], 'k.--', label="Equality of distribution", linewidth=2)
                plt.xlabel("Cumulative value of '"+var1+"'")
                plt.ylabel('Cumulative value of variables')
            fields = var2.split(";")
            try:
                # loop for X fields
                for i in range(len(fields)):
                    # cursor for sums of variables
                    stac_all, sec_all = 0, 0
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        ValueOfField = row.getValue(var1)
                        ValueOfField2 = row.getValue(fields[i])
                        stac_all = stac_all + ValueOfField
                        sec_all = sec_all + ValueOfField2
                    del row, rows
                    # cursor for density of row, write it into new field
                    rows = arcpy.UpdateCursor(in_features[p])
                    for row in rows:
                        dens = 0
                        stac = row.getValue(var1)
                        varia = row.getValue(fields[i])
                        if stac == 0:
                            dens = 0
                        else:
                            dens = varia/stac
                        row.setValue(dens_name, dens)
                        rows.updateRow(row)
                    del row, rows
                    # cursor to sort the density and for calculating proportion
                    prop_stac, prop_sec = [], [] # proportions of variables
                    rows = arcpy.UpdateCursor(in_features[p],"","","",dens_name)
                    for row in rows:
                        ValueOfField = row.getValue(var1)
                        ValueOfField2 = row.getValue(fields[i])
                        prop_stac.append(ValueOfField/stac_all)
                        prop_sec.append(ValueOfField2/sec_all)
                    del row, rows
                    # loop for cumulative proportions
                    xi, yi = [0.0], [0.0]
                    for j in range(len(prop_stac)):
                        if j == 0:
                            xi.append(prop_stac[j])
                            yi.append(prop_sec[j])
                        else:
                            xi.append(prop_stac[j]+xi[j])
                            yi.append(prop_sec[j]+yi[j])
                    # parts of gini index
                    sum1, sum2 = [], []
                    for k in range(len(xi)-1):
                        sum1.append((xi[k])*(yi[k+1]))
                    for k in range(len(yi)-1):
                        sum2.append((xi[k+1])*(yi[k]))
                    # gini index
                    GR = abs(sum(sum1)-sum(sum2))
                    arcpy.AddMessage("Gini index for "+ var1+" and "+ fields[i] +" is: "+str(GR) + " ("+str(in_features[p])+")")
                    # creating a plot lines
                    if parameters[3].value:
                        plt.plot(xi, yi, color=np.random.rand(3,1), label="Cumulative value of '"+fields[i]+"' (GI = "+str(round(GR,4))+") for '"+str(in_features[p])+"'", linewidth=2)

            except: raise
            finally:
                arcpy.DeleteField_management(in_features[p], dens_name)
                # create a plot when out_graph have a value
                if parameters[3].value:
                    plt.grid(True)
                    leg = plt.legend(loc=2,bbox_to_anchor=(1.02, 1), fancybox=True, shadow=True, borderaxespad=0.)
                    plt.savefig(out_graph+".png", bbox_extra_artists=(leg,), bbox_inches='tight')
                    plt.close()
        return

# -----======Centralization (Relative, Absolute centralization index)======-----
class Centralization(object):
    def __init__(self):
        self.label = "Centralization"
        self.description = ""
        self.canRunInBackground = True
        self.params = arcpy.GetParameterInfo()

    def getParameterInfo(self):
        in_indice = arcpy.Parameter(
			displayName="Select which index you want to calculate",
			name="in_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        in_indice.filter.type = "ValueList"
        in_indice.filter.list=["ACE - Absolute centralization index", "RCE - Relative centralization index"]

        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_pop_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Majority population field",
			name="majority_field",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        cent_indice = arcpy.Parameter(
			displayName="Customize central business district",
			name="cent_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        cent_indice.filter.type = "ValueList"
        cent_indice.filter.list=["Whole area centroid", "Customize centroid"]

        var4 = arcpy.Parameter(
			displayName="Central business district field",
			name="central_bus_district",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        var5 = arcpy.Parameter(
			displayName="Select central business polygon/point",
			name="select_central_bus_district",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Centrali"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_indice, in_features, var1, var2, cent_indice, var4, var5, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        in_indice = parameters[0].valueAsText
        cent_indice = parameters[4].valueAsText
        in_features = str(parameters[1].valueAsText)
        in_features = in_features.replace("\\\\","\\")
        count_features = in_features.split(";")
        if cent_indice == "Whole area centroid":
            parameters[5].enabled = False
            parameters[5].value = ""
            parameters[6].enabled = False
            parameters[6].value = ""
        existing = []
        if cent_indice == "Customize centroid":
            parameters[5].enabled = True
            parameters[6].enabled = True
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "String":
                        existing.append(field.name)
                parameters[5].filter.type = "ValueList"
                parameters[5].filter.list = sorted(existing)
        if in_indice == "ACE - Absolute centralization index":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = False
            parameters[3].value = ""
            parameters[4].enabled = True
        elif in_indice == "RCE - Relative centralization index":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = True
            parameters[4].enabled = True
        else:
            parameters[1].enabled = False
            parameters[2].enabled = False
            parameters[2].value = ""
            parameters[3].enabled = False
            parameters[3].value = ""
            parameters[4].enabled = False
            parameters[5].enabled = False
            parameters[6].enabled = False
        existing = []
        if parameters[1].value:
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
            parameters[3].filter.type = "ValueList"
            parameters[3].filter.list = sorted(existing)
        if len(count_features) > 1:
            parameters[4].enabled = False
            parameters[4].value = "Whole area centroid"
            parameters[5].enabled = False
            parameters[5].value = ""
            parameters[6].enabled = False
            parameters[6].value = ""
        if parameters[5].value:
            list_values = []
            rows = arcpy.SearchCursor(count_features[0])
            for row in rows:
                valueOfField = row.getValue(parameters[5].valueAsText)
                list_values.append(valueOfField)
            parameters[6].filter.type = "ValueList"
            parameters[6].filter.list = sorted(list_values)
        return

    def updateMessages(self, parameters):
        if len(parameters[7].valueAsText) > 10:
            parameters[7].setErrorMessage("The length of Field Name must not be larger than 10")
        if parameters[0].valueAsText == "RCE - Relative centralization index":
            if parameters[3].value is None:
                parameters[3].setErrorMessage("'Majority population field' must be filled!")
        if parameters[4].valueAsText == "Customize centroid":
            if parameters[5].value is None:
                parameters[5].setErrorMessage("'Central business district field' must be filled!")
            if parameters[6].value is None:
                parameters[6].setErrorMessage("'Select central business polygon/point' must be filled!")
        return
    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_indice = parameters[0].valueAsText
        in_features = parameters[1].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[2].valueAsText # minority
        var2 = parameters[3].valueAsText # majority
        cent_indice = parameters[4].valueAsText # indice of central business
        var4 = parameters[5].valueAsText # central business
        var5 = parameters[6].valueAsText
        field_out = parameters[7].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
             # calculation centroids
            x_cent = "X_centasi"
            arcpy.DeleteField_management(in_features[p], x_cent)
            arcpy.AddField_management(in_features[p], x_cent, "DOUBLE", "","", "", "X_centroid", "NULLABLE")
            arcpy.CalculateField_management(in_features[p], x_cent,"!SHAPE.CENTROID.X!","PYTHON_9.3")
            y_cent = "Y_centasi"
            arcpy.DeleteField_management(in_features[p], y_cent)
            arcpy.AddField_management(in_features[p], y_cent, "DOUBLE", "","", "", "Y_centroid", "NULLABLE")
            arcpy.CalculateField_management(in_features[p], y_cent,"!SHAPE.CENTROID.Y!","PYTHON_9.3")
            # add field for distance
            dist_r = "qwertydis"
            arcpy.DeleteField_management(in_features[p], dist_r)
            arcpy.AddField_management(in_features[p], dist_r, "DOUBLE", "","", "", "distace", "NULLABLE")
            # calculation distances between areas
            cxi,cyi,dij,cx,cy,j = 0,0,0,0,0,0
            cbd = [] # central busines district
            try:
                if cent_indice == "Whole area centroid":
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        cxi = row.getValue(x_cent)
                        cyi = row.getValue(y_cent)
                        cx = cx + cxi
                        cy = cy + cyi
                        j = j + 1
                    cbd = [cx/j,cy/j]
                if cent_indice == "Customize centroid":
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        sel_poly = row.getValue(var4)
                        if var5 == sel_poly:
                            cxi = row.getValue(x_cent)
                            cyi = row.getValue(y_cent)
                        else: continue
                        cbd = [cxi,cyi]
                rows2 = arcpy.UpdateCursor(in_features[p])
                for row2 in rows2:
                    cxi = row2.getValue(x_cent)
                    cyi = row2.getValue(y_cent)
                    dij = (math.sqrt(math.pow((cbd[0]-cxi),2)+math.pow((cbd[1]-cyi),2)))
                    row2.setValue(dist_r, dij)
                    rows2.updateRow(row2)
            except: raise
            finally:
                del row, rows
                arcpy.DeleteField_management(in_features[p], x_cent)
                arcpy.DeleteField_management(in_features[p], y_cent)

            # ------========ACE - Absolute centralization index========------
            if in_indice == "ACE - Absolute centralization index":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "AbsCentralIndex", "NULLABLE")
                area_r = "qwertyasd"
                arcpy.DeleteField_management(in_features[p], area_r)
                arcpy.AddField_management(in_features[p], area_r, "DOUBLE", "","", "", "area", "NULLABLE")
                arcpy.CalculateField_management(in_features[p], area_r, "!SHAPE.AREA@SQUAREMETERS!", "PYTHON_9.3")
            # ------========RCE - Relative centralization index========------
            elif in_indice == "RCE - Relative centralization index":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "RelCentralIndex", "NULLABLE")

            c_all = 0 # total minority population in regions
            a_all = 0 # total majority/area of regions
            Ci, Ai = [],[] # cumulative values
            c, pro_c = [],[]
            a, pro_a = [],[]
            RACE, RACE_1, RACE_2 = 0,0,0
            # adding minority and area into fields for next step
            try:
                # update cursor with sorting distace field - dist_r
                rows = arcpy.UpdateCursor(in_features[p],"","","",dist_r)
                for row in rows:
                    valueOfField = row.getValue(var1)
                    if in_indice == "ACE - Absolute centralization index":
                        valueOfField2 = row.getValue(area_r)
                    elif in_indice == "RCE - Relative centralization index":
                        valueOfField2 = row.getValue(var2)
                    c.append(valueOfField)
                    a.append(valueOfField2)
                c_all = sum(c)
                a_all = sum(a)
                # proportion
                for i in range(len(c)):
                    pro_c.append(c[i]/c_all)
                for i in range(len(a)):
                    pro_a.append(a[i]/a_all)
                # cumulative proportion
                for i in range(len(pro_c)):
                    if i == 0:
                        Ci.append(pro_c[i])
                    else:
                        Ci.append(pro_c[i]+Ci[i-1])
                for i in range(len(pro_a)):
                    if i == 0:
                        Ai.append(pro_a[i])
                    else:
                        Ai.append(pro_a[i]+Ai[i-1])
                # parts of formula to calculate ACE/RCE
                for i in range(len(Ci)):
                    RACE_1 = RACE_1 + (Ci[i-1]*Ai[i])
                    RACE_2 = RACE_2 + (Ci[i]*Ai[i-1])
                RACE = RACE_1 - RACE_2
                if in_indice == "ACE - Absolute centralization index":
                    arcpy.AddMessage("Absolute centralization index is: "+str(RACE)+ " ("+str(in_features[p])+")")
                elif in_indice == "RCE - Relative centralization index":
                    arcpy.AddMessage("Relative centralization index is: "+str(RACE)+ " ("+str(in_features[p])+")")
            except: raise
            finally:
                del row, rows
                arcpy.DeleteField_management(in_features[p], dist_r)
                arcpy.CalculateField_management(in_features[p], field_name, str(RACE), "PYTHON_9.3")
                if in_indice == "ACE - Absolute centralization index":
                    arcpy.DeleteField_management(in_features[p], area_r)
        return

# ---====Clustering (Index of Spatial proximity and of relative clustering)====---
class Clustering(object):
    def __init__(self):
        self.label = "Clustering"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_indice = arcpy.Parameter(
			displayName="Select which index you want to calculate",
			name="in_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        in_indice.filter.type = "ValueList"
        in_indice.filter.list=["Index of Spatial proximity", "Index of Relative clustering"]

        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_pop_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Majority population field",
			name="majority_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var3 = arcpy.Parameter(
			displayName="Total population field",
			name="total_population_field",
			datatype="String",
			parameterType="Optional",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Cluster"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_indice, in_features, var1, var2, var3, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        in_indice = parameters[0].valueAsText
        if in_indice == "Index of Spatial proximity":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = True
            parameters[4].enabled = True
        elif in_indice == "Index of Relative clustering":
            parameters[1].enabled = True
            parameters[2].enabled = True
            parameters[3].enabled = True
            parameters[4].enabled = False
            parameters[4].value = ""
        else:
            parameters[1].enabled = False
            parameters[2].enabled = False
            parameters[3].enabled = False
            parameters[4].enabled = False
        if parameters[1].value:
            in_features = parameters[1].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
            parameters[3].filter.type = "ValueList"
            parameters[3].filter.list = sorted(existing)
            parameters[4].filter.type = "ValueList"
            parameters[4].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[5].valueAsText) > 10:
            parameters[5].setErrorMessage("The length of Field Name must not be larger than 10")
        if parameters[0].valueAsText == "Index of Spatial proximity":
            if parameters[4].value is None:
                parameters[4].setErrorMessage("'Total population field' must be filled!")
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_indice = parameters[0].valueAsText
        in_features = parameters[1].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[2].valueAsText # minority
        var2 = parameters[3].valueAsText # majority
        var3 = parameters[4].valueAsText # total population
        field_out = parameters[5].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            # calculation centroids
            x_cent = "X_centasi"
            arcpy.DeleteField_management(in_features[p], x_cent)
            arcpy.AddField_management(in_features[p], x_cent, "DOUBLE", "","", "", "X_centroid", "NULLABLE")
            arcpy.CalculateField_management(in_features[p], x_cent,"!SHAPE.CENTROID.X!","PYTHON_9.3")
            y_cent = "Y_centasi"
            arcpy.DeleteField_management(in_features[p], y_cent)
            arcpy.AddField_management(in_features[p], y_cent, "DOUBLE", "","", "", "Y_centroid", "NULLABLE")
            arcpy.CalculateField_management(in_features[p], y_cent,"!SHAPE.CENTROID.Y!","PYTHON_9.3")
            # calculating total minority, majority, population
            X_all = 0 # total minority population
            Y_all = 0 # total majority population
            T_all = 0 # total population
            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    valueOfField = row.getValue(var1)
                    valueOfField2 = row.getValue(var2)
                    X_all = X_all + valueOfField
                    Y_all = Y_all + valueOfField2
                    if in_indice == "Index of Spatial proximity":
                        valueOfField3 = row.getValue(var3)
                        T_all = T_all + valueOfField3
            except: raise
            finally:
                if rows:
                    del row, rows
            # calculating average proximity between members
            xi,yi,ti,cxi,cyi,xj,yj,tj,cxj,cyj,dij,cij,Pxx,Pyy,Ptt = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    xi = row.getValue(var1)
                    yi = row.getValue(var2)
                    if in_indice == "Index of Spatial proximity":
                        ti = row.getValue(var3)
                    cxi = row.getValue(x_cent)
                    cyi = row.getValue(y_cent)
                    rows2 = arcpy.SearchCursor(in_features[p])
                    for row2 in rows2:
                        xj = row2.getValue(var1)
                        yj = row2.getValue(var2)
                        if in_indice == "Index of Spatial proximity":
                            tj = row2.getValue(var3)
                        cxj = row2.getValue(x_cent)
                        cyj = row2.getValue(y_cent)
                        dij = (math.sqrt(math.pow((cxj-cxi),2)+math.pow((cyj-cyi),2)))
                        cij = math.exp(-dij)
                        Pxx = Pxx + ((xi*xj*cij)/(math.pow(X_all,2)))
                        Pyy = Pyy + ((yi*yj*cij)/(math.pow(Y_all,2)))
                        if in_indice == "Index of Spatial proximity":
                            Ptt = Ptt + ((ti*tj*cij)/(math.pow(T_all,2)))
                    del row2,rows2
            except: raise
            finally:
                del row, rows
                arcpy.DeleteField_management(in_features[p], x_cent)
                arcpy.DeleteField_management(in_features[p], y_cent)

            # ------========Index of Spatial proximity========------
            if in_indice == "Index of Spatial proximity":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "ISpatialProximity", "NULLABLE")
                SP = 0
                SP = ((X_all*Pxx) + (Y_all*Pyy))/(T_all*Ptt)
                arcpy.AddMessage("Index of Spatial proximity is: "+str(SP)+ " ("+str(in_features[p])+")")
                arcpy.CalculateField_management(in_features[p], field_name, str(SP), "PYTHON_9.3")

            # ------========Index of Relative clustering========------
            if in_indice == "Index of Relative clustering":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "IRelativeClustering", "NULLABLE")
                RCL = 0
                RCL = (Pxx/Pyy) - 1
                arcpy.AddMessage("Index of Relative clustering is: "+str(RCL)+ " ("+str(in_features[p])+")")
                arcpy.CalculateField_management(in_features[p], field_name, str(RCL), "PYTHON_9.3")
        return
# ---====Neighbourhood (D(adj), D(w), D(s))====---
class Neighborhood(object):
    def __init__(self):
        self.label = "Neighborhood characteristics"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_indice = arcpy.Parameter(
			displayName="Select which index you want to calculate",
			name="in_indice",
			datatype="String",
			parameterType="Required",
			direction="Input")
        in_indice.filter.type = "ValueList"
        in_indice.filter.list=["D(adj) - Neighbor-adjusted index", "D(w) - Boundary-adjusted index", "D(s) - Index with compactness measure"]

        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_pop_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Majority population field",
			name="majority_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var3 = arcpy.Parameter(
			displayName="Population field",
			name="population",
			datatype="String",
			parameterType="Required",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Neighbor"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_indice, in_features, var1, var2, var3, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        if parameters[1].value:
            in_features = parameters[1].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
            parameters[3].filter.type = "ValueList"
            parameters[3].filter.list = sorted(existing)
            parameters[4].filter.type = "ValueList"
            parameters[4].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[5].valueAsText) > 10:
            parameters[5].setErrorMessage("The length of Field Name must not be larger than 10")
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_indice = parameters[0].valueAsText
        in_features = parameters[1].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[2].valueAsText # first population
        var2 = parameters[3].valueAsText # second population
        var3 = parameters[4].valueAsText # total popultion
        field_out = parameters[5].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")

            xi,yi,X,Y,Dis = 0,0,0,0,0 # Dissimilarity index
            # calculation of total populations
            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    valueOfField = row.getValue(var1)
                    valueOfField2 = row.getValue(var2)
                    X = X + valueOfField
                    Y = Y + valueOfField2
            # calculation of index
                rows2 = arcpy.SearchCursor(in_features[p])
                for row2 in rows2:
                    xi = row2.getValue(var1)
                    yi = row2.getValue(var2)
                    Dis = (Dis+abs(((xi/X)-(yi/Y))))
                Dis = Dis/2
            except: raise
            finally:
                del rows, row, rows2, row2

            if in_indice == "D(adj) - Neighbor-adjusted index":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "Neighbor-adjustedIndex", "NULLABLE")
                try:
                    sum_l,sum_u,Dadj = 0,0,0
                    # Getting geometry of polygons
                    desc = arcpy.Describe(in_features[p])
                    geom_f = desc.ShapeFieldName
                    # Starting calculating index...
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        geom1 = row.getValue(geom_f)
                        xi = row.getValue(var1)
                        #yi = row.getValue(var2)
                        ti = row.getValue(var3)
                        rows2 = arcpy.SearchCursor(in_features[p])
                        for row2 in rows2:
                            geom2 = row2.getValue(geom_f)
                            xj = row2.getValue(var1)
                            #yj = row2.getValue(var2)
                            tj = row2.getValue(var3)
                            # ...and finding neighbours
                            if geom1.touches(geom2) is True:
                                #if yi == 0 and yj == 0: sum_u = sum_u
                                if ti == 0 and tj != 0: sum_u = sum_u + abs(0-(xj/tj))
                                if ti != 0 and tj == 0: sum_u = sum_u + abs((xi/ti))
                                if ti != 0 and tj != 0: sum_u = sum_u + abs((xi/ti)-(xj/tj))
                                sum_l = sum_l + 1
                    Dadj = Dis - (sum_u/sum_l)
                    del row, rows, row2, rows2
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(Dadj), "PYTHON_9.3")
                    arcpy.AddMessage("Dissimilarity index is: " + str(Dis)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("Neighbor-adjusted D(adj) index is: " + str(Dadj)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("--------------------------------------------")

            if in_indice == "D(w) - Boundary-adjusted index":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "Boundary-adjustedIndex", "NULLABLE")
                try:
                    zizj,suma,Dw,dij,dij_i,wij,wij_zizj = 0,0,0,0,0,0,0
                    sum_dij = []
                    # Getting geometry of polygons
                    desc = arcpy.Describe(in_features[p])
                    geom_f = desc.ShapeFieldName
                    # calculating sum of dij
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        geom1 = row.getValue(geom_f)
                        rows2 = arcpy.SearchCursor(in_features[p])
                        for row2 in rows2:
                            geom2 = row2.getValue(geom_f)
                            if geom1.touches(geom2) is True:
                                inters = geom1.intersect(geom2,2)
                                dij = inters.getLength()
                                dij_i = dij_i + dij
                    del row, rows, row2, rows2

                    # Starting calculating index...
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        geom1 = row.getValue(geom_f)
                        xi = row.getValue(var1)
                        #yi = row.getValue(var2)
                        ti = row.getValue(var3)
                        rows2 = arcpy.SearchCursor(in_features[p])
                        for row2 in rows2:
                            geom2 = row2.getValue(geom_f)
                            xj = row2.getValue(var1)
                            #yj = row2.getValue(var2)
                            tj = row2.getValue(var3)
                            # ...and finding neighbours
                            if geom1.touches(geom2) is True:
                                zizj = 0
                                inters = geom1.intersect(geom2,2)
                                dij = inters.getLength()
                                wij = dij/dij_i
                                if ti == 0 and tj == 0: zizj = 0
                                if ti == 0 and tj != 0: zizj = abs(0-(xj/tj))
                                if ti != 0 and tj == 0: zizj = abs((xi/ti))
                                if ti != 0 and tj != 0: zizj = abs((xi/ti)-(xj/tj))
                                wij_zizj = wij_zizj + (wij*zizj)
                    Dw = Dis - (wij_zizj/2)
                    del row, rows, row2, rows2
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(Dw), "PYTHON_9.3")
                    arcpy.AddMessage("Dissimilarity index is: " + str(Dis)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("Boundary-adjusted D(w) index is: " + str(Dw)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("--------------------------------------------")

            if in_indice == "D(s) - Index with compactness measure":
                arcpy.DeleteField_management(in_features[p], field_name)
                arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "IndexCompactness", "NULLABLE")
                area_r = "areaqwert"
                arcpy.DeleteField_management(in_features[p], area_r)
                arcpy.AddField_management(in_features[p], area_r, "DOUBLE", "","", "", "area", "NULLABLE")
                arcpy.CalculateField_management(in_features[p], area_r, "!SHAPE.AREA@SQUAREMETERS!", "PYTHON_9.3")
                peri_r = "periqwert"
                arcpy.DeleteField_management(in_features[p], peri_r)
                arcpy.AddField_management(in_features[p], peri_r, "DOUBLE", "","", "", "area", "NULLABLE")
                arcpy.CalculateField_management(in_features[p], peri_r, "!SHAPE.LENGTH@METERS!", "PYTHON_9.3")

                try:
                    zizj,Ds,dij,dij_i,wij,wij_zizj,P_A = 0,0,0,0,0,0,0
                    sum_P_A = []
                    # Getting geometry of polygons
                    desc = arcpy.Describe(in_features[p])
                    geom_f = desc.ShapeFieldName
                    # calculating sum of dij
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        geom1 = row.getValue(geom_f)
                        area_i = row.getValue(area_r)
                        peri_i = row.getValue(peri_r)
                        sum_P_A.append(peri_i/area_i)
                        rows2 = arcpy.SearchCursor(in_features[p])
                        for row2 in rows2:
                            geom2 = row2.getValue(geom_f)
                            if geom1.touches(geom2) is True:
                                inters = geom1.intersect(geom2,2)
                                dij = inters.getLength()
                                dij_i = dij_i + dij
                    del row, rows, row2, rows2
                    # Starting calculating index...
                    rows = arcpy.SearchCursor(in_features[p])
                    for row in rows:
                        geom1 = row.getValue(geom_f)
                        xi = row.getValue(var1)
                        #yi = row.getValue(var2)
                        ti = row.getValue(var3)
                        Ai = row.getValue(area_r)
                        Pi = row.getValue(peri_r)
                        rows2 = arcpy.SearchCursor(in_features[p])
                        for row2 in rows2:
                            geom2 = row2.getValue(geom_f)
                            xj = row2.getValue(var1)
                            #yj = row2.getValue(var2)
                            tj = row2.getValue(var3)
                            Aj = row2.getValue(area_r)
                            Pj = row2.getValue(peri_r)
                            # ...and finding neighbours
                            if geom1.touches(geom2) is True:
                                zizj = 0
                                inters = geom1.intersect(geom2,2)
                                dij = inters.getLength()
                                wij = dij/dij_i
                                if ti == 0 and tj == 0: zizj = 0
                                if ti == 0 and tj != 0: zizj = abs(0-(xj/tj))
                                if ti != 0 and tj == 0: zizj = abs((xi/ti))
                                if ti != 0 and tj != 0: zizj = abs((xi/ti)-(xj/tj))
                                P_A = (((Pi/Ai)+(Pj/Aj))/2)/(max(sum_P_A))
                                wij_zizj = wij_zizj + (wij*zizj*P_A)
                    Ds = Dis - (wij_zizj/2)
                    del row, rows, row2, rows2
                except: raise
                finally:
                    arcpy.CalculateField_management(in_features[p], field_name, str(Ds), "PYTHON_9.3")
                    arcpy.AddMessage("Dissimilarity index is: " + str(Dis)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("Index D(s) with compactness measure is: " + str(Ds)+ " ("+str(in_features[p])+")")
                    arcpy.AddMessage("--------------------------------------------")
                    arcpy.DeleteField_management(in_features[p], area_r)
                    arcpy.DeleteField_management(in_features[p], peri_r)
        return
# --------===========Segregation index===========--------
class Segregation(object):
    def __init__(self):
        self.label = "Segregation index"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        in_features = arcpy.Parameter(
			displayName="Input Features",
			name="in_features",
			datatype="Feature Layer",
			parameterType="Required",
			direction="Input",
            multiValue=True)
        in_features.filter.list=["Polygon","Point"]

        var1 = arcpy.Parameter(
			displayName="Minority population field",
			name="minority_population_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        var2 = arcpy.Parameter(
			displayName="Population field",
			name="population_field",
			datatype="String",
			parameterType="Required",
			direction="Input")

        field_out = arcpy.Parameter(
			displayName="Result column name",
			name="result_column_name",
			datatype="String",
			parameterType="Required",
			direction="Input")
        field_out.value = "Segreg"

        out_features = arcpy.Parameter(
			displayName="Output Features",
			name="out_features",
			datatype="Feature Layer",
			parameterType="Derived",
			direction="Output")
        out_features.parameterDependencies=[in_features.name]
        out_features.schema.clone = True

        params = [in_features, var1, var2, field_out, out_features]
        return params

    def isLicensed(self):
        return True
    def updateParameters(self, parameters):
        if parameters[0].value:
            in_features = parameters[0].valueAsText
            existing = []
            count_features = in_features.split(";")
            for i in range(len(count_features)):
                for field in arcpy.ListFields(count_features[i]):
                    if field.type == "Double" or field.type == "Integer" or field.type == "SmallInteger" or field.type == "Single":
                        existing.append(field.name)
                if i > 0:
                    existing = list(set([x for x in existing if existing.count(x) > 1]))
            parameters[1].filter.type = "ValueList"
            parameters[1].filter.list = sorted(existing)
            parameters[2].filter.type = "ValueList"
            parameters[2].filter.list = sorted(existing)
        return
    def updateMessages(self, parameters):
        if len(parameters[3].valueAsText) > 10:
            parameters[3].setErrorMessage("The length of Field Name must not be larger than 10")
        return
    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_features = parameters[0].valueAsText
        in_features = in_features.split(";")
        var1 = parameters[1].valueAsText # minority population
        var2 = parameters[2].valueAsText # population
        field_out = parameters[3].valueAsText
        field_name = field_out.replace(" ","_")
        for p in range(len(in_features)):
            arcpy.SelectLayerByAttribute_management(in_features[p], "CLEAR_SELECTION")
            arcpy.DeleteField_management(in_features[p], field_name)
            arcpy.AddField_management(in_features[p], field_name, "DOUBLE", "","", "", "DissimilarityIndex", "NULLABLE")
            xi,yi,X,Y,dis,seg = 0,0,0,0,0,0
            # calculation of total populations
            try:
                rows = arcpy.SearchCursor(in_features[p])
                for row in rows:
                    valueOfField = row.getValue(var1)
                    valueOfField2 = row.getValue(var2)
                    X = X + valueOfField
                    Y = Y + valueOfField2
            # calculation of index
                rows2 = arcpy.SearchCursor(in_features[p])
                for row2 in rows2:
                    xi = row2.getValue(var1)
                    yi = row2.getValue(var2)
                    dis = (dis+abs((xi/X)-(yi/Y)))
                dis = dis/2
                seg = dis/(1-(X/Y))
                arcpy.AddMessage("Segregation index is: "+ str(seg)+ " ("+str(in_features[p])+")")
            except: raise
            finally:
                arcpy.CalculateField_management(in_features[p], field_name, str(seg), "PYTHON_9.3")
                del rows, row, rows2, row2
        return